#include<iostream>
#include<stdlib.h>
#include<string.h>
#include"row.h"


// The function "chromatic" reads the accidental and the head of a
// numerical note and returns the chromatic number of it, e.g. ^f
// (1,3)=> 6, _a (-1,5) => 8, _b (-1,-1) or (-1 -8)=>10. The range at
// "small" is one octave (0-11) and at "big" the range is the analog.
// x=accidental, y=head.

int Row::chromatic_big(int x, int y){
  if(y%7<3) return(2*(y%7)+x+12*(y/7));
  else return(2*(y%7)+x-1+12*(y/7));
}

int Row::chromatic_big(Note One){
  int x=One.getAccid(), y=One.getHead();
  if(y%7<3) return(2*(y%7)+x+12*(y/7));
  else return(2*(y%7)+x-1+12*(y/7));
}

void Row::smooth(Note Basi){
  for(int i=0; i<itsPlithos; i++){
    if(itsNotes[i].getHead()>Basi.getHead()+6)
      itsNotes[i].setHead(itsNotes[i].getHead()-7);
    else if(itsNotes[i].getHead()<Basi.getHead())
      itsNotes[i].setHead(itsNotes[i].getHead()+7);
  }
}

void Row::climax(void){
  int i, j, note1, note2;
  Note Temp;

  for(i=0; i<itsPlithos; i++){
    for(j=0; j<itsPlithos-i-1; j++){
      note1=chromatic_big(itsNotes[j]);
      note2=chromatic_big(itsNotes[j+1]);
	if(note1>note2){
	  Temp=itsNotes[j+1];
	  setNotes(j+1, itsNotes[j]);
	  setNotes(j, Temp);
	}
	else if(note1==note2) setNotes(j, itsNotes[j+1]);
    }
  }
}

void Row::setNotes(int position, Note One){
  itsNotes[position].setAccid(One.getAccid());
  itsNotes[position].setHead(One.getHead());
}

void Row::setPlithos(int plithos){
  itsPlithos=plithos;
}

bool Row::input_code(char arxi){
  return(arxi=='^' || arxi=='=' || arxi=='_');
}

void Row::tex2num(void){
  for(int i=0; i<itsPlithos; i++)
    itsNotes[i].tex2num();
}

void Row::german2num(void){
  for(int i=0; i<itsPlithos; i++)
    itsNotes[i].german2num();
}

void Row::set_char_gRow(void){
  for(int i=0; i<itsPlithos; i++)
    itsNotes[i].set_gNote(sep_notes[i]);
}

void Row::set_char_TeXRow(void){
  for(int i=0; i<itsPlithos; i++)
    itsNotes[i].set_TeXNote(sep_notes[i]);
}

void Row::divide(char *line){
  char line2[100];
  int i=0;
  char tokensep[]=" \t,\n", *note;
  strcpy(line2, line);
  note=strtok(line, tokensep);
  while(note!=NULL){
    strcpy(sep_notes[i], note);
    i++;
    note=strtok(NULL, tokensep);
  }
  itsPlithos=i;
  strcpy(line, line2);
}

void Row::make(char *line){
  int i;
  divide(line); // it creates itsPlithos and sep_notes[]
  if(check4tex()){
    set_char_TeXRow();
    for(i=0; i<itsPlithos; i++)
      itsNotes[i].tex2num();
  }
  else{
    set_char_gRow();
    for(i=0; i<itsPlithos; i++)
      itsNotes[i].german2num();
  }

}

int Row::check4tex(void){
  int i;
  for(i=0; i<itsPlithos; i++){
    if(input_code(sep_notes[i][0])) return 1;
  }
  if(i==itsPlithos) return 0;
}

void Row::setNotes(int num_notes[][2]){
  for(int i=0; i<itsPlithos; i++){
    itsNotes[i].setAccid(num_notes[i][0]);
    itsNotes[i].setHead(num_notes[i][1]);
  }
}

Note Row::getNotes(int position){
  return itsNotes[position];
}

void Row::transpose(Interval Diastima){
  for(int i=0; i<itsPlithos; i++){
    itsNotes[i].transpose(Diastima);
  }
}

void Row::retrograde(void){
  Note Temp[24];
  int i;
  for(i=0; i<itsPlithos; i++)
    Temp[i]=itsNotes[i];
  for(i=0; i<itsPlithos; i++)
    itsNotes[i]=Temp[itsPlithos-i-1];
}

void Row::enharmonic(int i){
  itsNotes[i].enharmonic();
}

void Row::invert(void){
  int i;
  Interval Diastima;
  Row Temp;
  for(i=0; i<itsPlithos; i++){
    Temp.itsNotes[i].setAccid(itsNotes[0].getAccid());
    Temp.itsNotes[i].setHead(itsNotes[0].getHead());
  }
  for(i=1; i<itsPlithos; i++){
    if(recognize(getNotes(0), getNotes(i), Diastima)==0){
      enharmonic(i);
      recognize(getNotes(0), getNotes(i), Diastima);
    }
    Diastima.set_space(0-Diastima.get_space());
    Temp.itsNotes[i].transpose(Diastima);
    itsNotes[i]=Temp.itsNotes[i];
  }
}

int Row::chr_interval(Note One, Note Two){
return(chromatic_big(Two.getAccid(), Two.getHead())
	-chromatic_big(One.getAccid(), One.getHead()));
}

// "recognize" reads two notes, gives the melodic interval and returns
// 1. If the interval does not exist, it returns 0.
int Row::recognize(Note One, Note Two, Interval& Diastima){
  int d_diff, chr_diff; // diatonic and chromatic interval
  int ad_interval; //absolute diatonic interval

  d_diff=Two.getHead()-One.getHead();
  if(d_diff) Diastima.set_space(d_diff+abs(d_diff)/d_diff);
  else{
      if(chr_interval(One, Two)>=0) Diastima.set_space(1);
      else Diastima.set_space(-1);
  }
  ad_interval=abs(Diastima.get_space());
  chr_diff=abs(chr_interval(One, Two));
  if((ad_interval%7==2 && chr_diff%12==0) ||
     (ad_interval%7==3 && chr_diff%12==2) ||
     (ad_interval%7==4 && chr_diff%12==4) ||
     (ad_interval%7==5 && chr_diff%12==6) ||
     (ad_interval%7==6 && chr_diff%12==7) ||
     (ad_interval%7==0 && chr_diff%12==9) ||
     (ad_interval%7==1 && chr_diff%12==11))
    Diastima.set_type('d');
  else if((ad_interval%7==2 && chr_diff%12==1) ||
	  (ad_interval%7==3 && chr_diff%12==3) ||
	  (ad_interval%7==6 && chr_diff%12==8) ||
	  (ad_interval%7==0 && chr_diff%12==10))
    Diastima.set_type('m');
  else if((ad_interval%7==2 && chr_diff%12==2) ||
	  (ad_interval%7==3 && chr_diff%12==4) ||
	  (ad_interval%7==6 && chr_diff%12==9) ||
	  (ad_interval%7==0 && chr_diff%12==11))
    Diastima.set_type('M');
  else if((ad_interval%7==4 && chr_diff%12==5) ||
	  (ad_interval%7==5 && chr_diff%12==7) ||
	  (ad_interval%7==1 && chr_diff%12==0))
    Diastima.set_type('p');
  else if((ad_interval%7==2 && chr_diff%12==3) ||
	  (ad_interval%7==3 && chr_diff%12==5) ||
	  (ad_interval%7==4 && chr_diff%12==6) ||
	  (ad_interval%7==5 && chr_diff%12==8) ||
	  (ad_interval%7==6 && chr_diff%12==10) ||
	  (ad_interval%7==0 && chr_diff%12==0) ||
	  (ad_interval%7==1 && chr_diff%12==1))
    Diastima.set_type('a');
  else{
    Diastima.set_space(0);
    Diastima.set_type('x');
    return 0;
  }
  return 1;
}

int Row::getPlithos(void){
  return itsPlithos;
}

// Filter for triple accidentals
void Row::filter3(void){
  for(int i=0; i<itsPlithos; i++)
    if(abs(itsNotes[i].getAccid())==3) itsNotes[i].enharmonic();
}

// Filter for double accidentals
void Row::filter2(void){
  for(int i=0; i<itsPlithos; i++)
    if(abs(itsNotes[i].getAccid())==2) itsNotes[i].enharmonic();
}

// Filter for e/b sharp, c/f flat
void Row::filter1(void){
  for(int i=0; i<itsPlithos; i++){
    if(itsNotes[i].getAccid()==1){
      if(itsNotes[i].getHead()%7==2 || itsNotes[i].getHead()%7==6)
	itsNotes[i].enharmonic();
    }
    else if(itsNotes[i].getAccid()==-1){
      if(itsNotes[i].getHead()%7==0 || itsNotes[i].getHead()%7==3)
	itsNotes[i].enharmonic();
    }
  }
}

void Row::ambitus(void){
  int i, d;

  for(i=0; i<itsPlithos; i++){
    if(itsNotes[i].getHead()<21){
      d=-1; break;
    }
    else if(itsNotes[i].getHead()>34){
      d=1; break;
    }
    else d=0;
  }

  if(d!=0){
    for(i=0; i<itsPlithos; i++)
      itsNotes[i].setHead(itsNotes[i].getHead()-7*d);
  }
}
